@@ -109,7 +109,7 @@ rSobol <- function(n, ranges) {
   upper <- matrix( rep(ranges[,2], npart), nrow=npart, byrow=TRUE)
   
   # Sobol initialization for all the particles, with a value in [0,1]
-  X <- randtoolbox::sobol(n=n, dim=ndim, scrambling=3, seed = floor(runif(1, 1, 99999)))  
+  X <- randtoolbox::sobol(n = n, dim = ndim)
   
   # Transforming X into the real range defined by the user
   #X <- t( lower +  (upper - lower )*t(X) ) # when using vector instead of matrixes
