@@ -1,15 +1,15 @@
-###############################################################################################
-##                                        hydroMOPSO                                         ##
-###############################################################################################
-# Author : Rodrigo Marinao Rivas                                                             ##
-###############################################################################################
-# Created: during 2020                                                                       ##
-###############################################################################################
+################################################################################
+##                                 hydroMOPSO                                 ##
+################################################################################
+# Author : Rodrigo Marinao Rivas                                              ##
+################################################################################
+# Created: during 2020                                                        ##
+################################################################################
 # Main references: Lin et al. (2018); Marinao-Rivas & Zambrano-Bigiarini (2021)
 #
-# Description: Multi-objective Particle Swarm Optimisation algorithm (NMPSO). The default 
-# configuration of hydroMOPSO has been adapted to obtain results with the fewest number of 
-# iterations possible.
+# Description: Multi-objective Particle Swarm Optimisation algorithm (NMPSO). 
+#              The default configuration of hydroMOPSO has been adapted to 
+#              obtain results with the fewest number of iterations possible.
 #
 # Function and Input arguments
 hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a character or a function indicating the function (forgive the 
@@ -24,6 +24,7 @@ hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a
                                              # representing the model code to be calibrated/optimised
                       model.FUN.args=list(), # >> 'list' (only used only when fn='hydromod' or fn='hydromodInR'). List with the arguments to pass
                                              # to model.FUN
+                      BC.space.norm = FALSE, 
                       obj.thr = NULL,
                       ...                    # Further arguments to pass to fn, if needed
                       ){ 
@@ -216,7 +217,7 @@ hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a
       message("[                          Reading 'param.ranges' ...                          ]")
     if (verbose) 
       message("================================================================================")
-    X.Boundaries <- ReadParamRanges(ParamRanges.fname = param.ranges)
+    X.Boundaries <- read.ParameterRanges(ParamRanges.fname = param.ranges)
     lower <- X.Boundaries[, 1]
     upper <- X.Boundaries[, 2]
   }else {
@@ -707,14 +708,15 @@ hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a
 
       df.obj.thr <- data.frame(matrix(NA, ncol = nobj, nrow = 2))
       colnames(df.obj.thr) <- paste0("Obj",seq(1,nobj))
-      df.obj.thr[1,1:nobj] <- ifelse(is.null(obj.thr), rep(NA, nobj), sapply(obj.thr,"[",1))
-      df.obj.thr[2,1:nobj] <- ifelse(is.null(obj.thr), rep(NA, nobj), sapply(obj.thr,"[",2))
+      df.obj.thr[1,1:nobj] <- if(is.null(obj.thr)) rep(NA, nobj) else sapply(obj.thr,"[",1)
+      df.obj.thr[2,1:nobj] <- if(is.null(obj.thr)) rep(NA, nobj) else sapply(obj.thr,"[",2)
 
 
       if(write2disk){
 
         write.table(df.minmax, paste0(drty.out, "/MOPSO_MaxMin.txt"), row.names = FALSE, quote = FALSE)
         write.table(df.obj.names, paste0(drty.out, "/MOPSO_ObjsNames.txt"), row.names = FALSE, quote = FALSE)
+        write.table(data.frame("flag" = BC.space.norm), paste0(drty.out, "/MOPSO_BCSpaceNorm.txt"), row.names = FALSE, quote = FALSE)
         write.table(df.obj.thr, paste0(drty.out, "/MOPSO_ObjsThresholds.txt"), row.names = FALSE, quote = FALSE)
 
       }
@@ -741,8 +743,6 @@ hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a
           df.var.units[1,1:nvar] <- var.units[1:nvar]
         }
 
-        df.vars <- cbind.data.frame(df.var.names, df.var.units)
-
 
         # warm up
 
@@ -771,7 +771,8 @@ hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a
         if(write2disk){
 
           write.table(df.dimensions, paste0(drty.out, "/hydro_Dimensions.txt"), row.names = FALSE, quote = FALSE)
-          write.table(df.vars, paste0(drty.out, "/hydro_NamesAndUnitsVars.txt"), row.names = FALSE, quote = FALSE)
+          write.table(df.var.names, paste0(drty.out, "/hydro_VarNames.txt"), row.names = FALSE, quote = FALSE)
+          write.table(df.var.units, paste0(drty.out, "/hydro_VarUnits.txt"), row.names = FALSE, quote = FALSE)
           write.table(df.warmup, paste0(drty.out, "/hydro_DatesWarmUp.txt"), row.names = FALSE, quote = FALSE)
           write.table(df.dates.cal, paste0(drty.out, "/hydro_DatesCal.txt"), row.names = FALSE, quote = FALSE)
 
@@ -1366,7 +1367,7 @@ hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a
     close(NMPSOparam.TextFile)
     
 
-  
+
     if ( (fn.name=="hydromod") | (fn.name=="hydromodInR") ) {
       
       hydroMOPSOparam.TextFile <- file(hydroMOPSOparam.fname, "a")    
@@ -1413,8 +1414,31 @@ hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a
                        "ParticlesParetoFront" = pos.history,
                        "ObjsNames" = df.obj.names,
                        "MaxMin" = df.minmax,
-                       "ObjsThreshold" = df.obj.thr
+                       "BCSpaceNorm" = BC.space.norm,
+                       "ObjsThresholds" = df.obj.thr
                        )
+  
+  
+  MOPSOArgs <- list(
+                    fn = fn,
+                    lower = lower,
+                    upper = upper,
+                    control = control,
+                    model.FUN = model.FUN,
+                    model.FUN.args = model.FUN.args,
+                    BC.space.norm = BC.space.norm,
+                    obj.thr = obj.thr
+                  )
+  
+  
+  if(write2disk){
+    MOPSOArgs.fname <- paste(file.path(drty.out),
+                             "/",
+                             "hydroMOPSO_args.rds", sep="")
+  
+    saveRDS(MOPSOArgs, file = MOPSOArgs.fname)
+    
+  }
 
 
   if(fn.name %in% c("hydromod", "hydromodInR")){
@@ -1422,7 +1446,8 @@ hydroMOPSO <-function(fn="hydromod",         # >> 'character/function'. Either a
 
     hydroDetails <- list("Obs" = list.obs, # hyd
                          "Dimensions" = df.dimensions, # hyd
-                         "NamesAndUnitsVars" = df.vars, # hyd
+                         "VarNames" = df.var.names, # hyd
+                         "VarUnits" = df.var.units, # hyd
                          "WarmUp" = df.warmup, # hyd
                          "DatesCal" = df.dates.cal # hyd
                          )
